@@ -79,7 +79,6 @@ import org.apache.commons.math3.exception.util.LocalizedFormats;
  * requirement, throwing a {@code MathIllegalArgumentException} if it is
  * violated.
  * </p>
- * @version $Id: ResizableDoubleArray.java 1591835 2014-05-02 09:04:01Z tn $
  */
 public class ResizableDoubleArray implements DoubleArray, Serializable {
     /** Additive expansion mode.
@@ -412,7 +411,7 @@ public class ResizableDoubleArray implements DoubleArray, Serializable {
         numElements = 0;
         startIndex = 0;
 
-        if (data != null) {
+        if (data != null && data.length > 1) {
             addElements(data);
         }
     }
