@@ -17,17 +17,16 @@
 
 package org.apache.commons.math3.complex;
 
+import java.util.List;
+
 import org.apache.commons.math3.TestUtils;
 import org.apache.commons.math3.exception.NullArgumentException;
 import org.apache.commons.math3.util.FastMath;
 import org.junit.Assert;
 import org.junit.Test;
 
-import java.util.List;
-
 
 /**
- * @version $Id: ComplexTest.java 1588754 2014-04-20 13:25:36Z luc $
  */
 public class ComplexTest {
 
@@ -782,14 +781,83 @@ public class ComplexTest {
     }
 
     @Test
-    public void testExpInf() {
+    public void testExpInf1() {
         TestUtils.assertSame(Complex.NaN, oneInf.exp());
+    }
+
+    @Test
+    public void testExpInf2() {
         TestUtils.assertSame(Complex.NaN, oneNegInf.exp());
+    }
+
+    @Test
+    public void testExpInf3() {
         TestUtils.assertSame(infInf, infOne.exp());
-        TestUtils.assertSame(Complex.ZERO, negInfOne.exp());
+    }
+
+    @Test
+    public void testJava() {// TODO more debug
+        System.out.println(">>testJava()");
+        // FastMathTest#testExpSpecialCases() checks the following:
+        // Assert.assertEquals("exp of -infinity should be 0.0", 0.0, FastMath.exp(Double.NEGATIVE_INFINITY), Precision.EPSILON);
+        // Let's check how well Math works:
+        System.out.println("Math.exp="+Math.exp(Double.NEGATIVE_INFINITY));
+        String props[] = {
+        "java.version", //    Java Runtime Environment version
+        "java.vendor", // Java Runtime Environment vendor
+        "java.vm.specification.version", //   Java Virtual Machine specification version
+        "java.vm.specification.vendor", //    Java Virtual Machine specification vendor
+        "java.vm.specification.name", //  Java Virtual Machine specification name
+        "java.vm.version", // Java Virtual Machine implementation version
+        "java.vm.vendor", //  Java Virtual Machine implementation vendor
+        "java.vm.name", //    Java Virtual Machine implementation name
+        "java.specification.version", //  Java Runtime Environment specification version
+        "java.specification.vendor", //   Java Runtime Environment specification vendor
+        "java.specification.name", // Java Runtime Environment specification name
+        "java.class.version", //  Java class format version number
+        };
+        for(String t : props) {
+            System.out.println(t + "=" + System.getProperty(t));
+        }    
+        System.out.println("<<testJava()");
+    }
+
+    @Test
+    public void testExpInf4() {
+        // TODO: temp debug start
+        System.out.println(">>testExpInf4()");
+        double inf = negInfOne.getReal();
+        System.out.println("inf="+inf);
+        int intVal = (int) -inf;
+        System.out.println("intval="+intVal);
+        System.out.println("neginf<0=" + (inf < 0.0));
+        // TODO: temp debug end
+        System.out.println("expReal="+FastMath.exp(negInfOne.getReal())); // TODO temp debug
+        System.out.println("cosImag="+FastMath.cos(negInfOne.getImaginary())); // TODO temp debug
+        System.out.println("sinImag="+FastMath.sin(negInfOne.getImaginary())); // TODO temp debug
+        final Complex exp = negInfOne.exp();
+        System.out.println("result="+exp); // TODO temp debug
+        TestUtils.assertSame(Complex.ZERO, exp);
+        System.out.println("<<testExpInf4()"); // TODO debug
+    }
+
+    @Test
+    public void testExpInf5() {
         TestUtils.assertSame(Complex.NaN, infInf.exp());
+    }
+
+    @Test
+    public void testExpInf6() {
         TestUtils.assertSame(Complex.NaN, infNegInf.exp());
+    }
+
+    @Test
+    public void testExpInf7() {
         TestUtils.assertSame(Complex.NaN, negInfInf.exp());
+    }
+
+    @Test
+    public void testExpInf8() {
         TestUtils.assertSame(Complex.NaN, negInfNegInf.exp());
     }
 
