@@ -28,13 +28,12 @@ import org.apache.commons.math3.TestUtils;
 import org.apache.commons.math3.analysis.UnivariateFunction;
 import org.apache.commons.math3.analysis.integration.BaseAbstractUnivariateIntegrator;
 import org.apache.commons.math3.analysis.integration.IterativeLegendreGaussIntegrator;
-import org.apache.commons.math3.distribution.AbstractRealDistribution;
+import org.apache.commons.math3.distribution.ConstantRealDistribution;
 import org.apache.commons.math3.distribution.NormalDistribution;
 import org.apache.commons.math3.distribution.RealDistribution;
 import org.apache.commons.math3.distribution.RealDistributionAbstractTest;
 import org.apache.commons.math3.distribution.UniformRealDistribution;
 import org.apache.commons.math3.exception.NullArgumentException;
-import org.apache.commons.math3.exception.OutOfRangeException;
 import org.apache.commons.math3.stat.descriptive.SummaryStatistics;
 import org.apache.commons.math3.util.FastMath;
 import org.junit.Assert;
@@ -44,7 +43,6 @@ import org.junit.Test;
 /**
  * Test cases for the EmpiricalDistribution class
  *
- * @version $Id: EmpiricalDistributionTest.java 1547633 2013-12-03 23:03:06Z tn $
  */
 
 public final class EmpiricalDistributionTest extends RealDistributionAbstractTest {
@@ -409,6 +407,50 @@ public final class EmpiricalDistributionTest extends RealDistributionAbstractTes
         }
     }
     
+    /** 
+     * MATH-984
+     * Verify that sampled values do not go outside of the range of the data.
+     */
+    @Test
+    public void testSampleValuesRange() {
+        // Concentrate values near the endpoints of (0, 1).
+        // Unconstrained Gaussian kernel would generate values outside the interval.
+        final double[] data = new double[100];
+        for (int i = 0; i < 50; i++) {
+            data[i] = 1 / ((double) i + 1);
+        }
+        for (int i = 51; i < 100; i++) {
+            data[i] = 1 - 1 / (100 - (double) i + 2);
+        }
+        EmpiricalDistribution dist = new EmpiricalDistribution(10);
+        dist.load(data);
+        dist.reseedRandomGenerator(1000);
+        for (int i = 0; i < 1000; i++) {
+            final double dev = dist.sample();
+            Assert.assertTrue(dev < 1);
+            Assert.assertTrue(dev > 0);
+        }
+    }
+    
+    /**
+     * MATH-1203, MATH-1208
+     */
+    @Test
+    public void testNoBinVariance() {
+        final double[] data = {0, 0, 1, 1};
+        EmpiricalDistribution dist = new EmpiricalDistribution(2);
+        dist.load(data);
+        dist.reseedRandomGenerator(1000);
+        for (int i = 0; i < 1000; i++) {
+            final double dev = dist.sample();
+            Assert.assertTrue(dev == 0 || dev == 1);
+        }
+        Assert.assertEquals(0.5, dist.cumulativeProbability(0), Double.MIN_VALUE);
+        Assert.assertEquals(1.0, dist.cumulativeProbability(1), Double.MIN_VALUE);
+        Assert.assertEquals(0.5, dist.cumulativeProbability(0.5), Double.MIN_VALUE);
+        Assert.assertEquals(0.5, dist.cumulativeProbability(0.7), Double.MIN_VALUE);
+    }   
+    
     /**
      * Find the bin that x belongs (relative to {@link #makeDistribution()}).
      */
@@ -503,7 +545,7 @@ public final class EmpiricalDistributionTest extends RealDistributionAbstractTes
         // Use constant distribution equal to bin mean within bin
         @Override
         protected RealDistribution getKernel(SummaryStatistics bStats) {
-            return new ConstantDistribution(bStats.getMean());
+            return new ConstantRealDistribution(bStats.getMean());
         }
     }
     
@@ -521,68 +563,4 @@ public final class EmpiricalDistributionTest extends RealDistributionAbstractTes
                     UniformRealDistribution.DEFAULT_INVERSE_ABSOLUTE_ACCURACY);
         }
     }
-    
-    /**
-     * Distribution that takes just one value.
-     */
-    private class ConstantDistribution extends AbstractRealDistribution {
-        private static final long serialVersionUID = 1L;
-        
-        /** Singleton value in the sample space */
-        private final double c;
-        
-        public ConstantDistribution(double c) {
-            this.c = c;
-        }
-        
-        public double density(double x) {
-            return 0;
-        }
-
-        public double cumulativeProbability(double x) {
-            return x < c ? 0 : 1;
-        }
-        
-        @Override
-        public double inverseCumulativeProbability(double p) {
-            if (p < 0.0 || p > 1.0) {
-                throw new OutOfRangeException(p, 0, 1);
-            }
-            return c;
-        }
-
-        public double getNumericalMean() {
-            return c;
-        }
-
-        public double getNumericalVariance() {
-            return 0;
-        }
-
-        public double getSupportLowerBound() {
-            return c;
-        }
-
-        public double getSupportUpperBound() {
-            return c;
-        }
-
-        public boolean isSupportLowerBoundInclusive() {
-            return false;
-        }
-
-        public boolean isSupportUpperBoundInclusive() {
-            return true;
-        }
-
-        public boolean isSupportConnected() {
-            return true;
-        }
-        
-        @Override
-        public double sample() {
-            return c;
-        }
-        
-    }
 }
